# SPDX-License-Identifier: CC0-1.0


class SequentialFiles:
    def __init__(self, filenames, sizes, mode=None):
        assert len(filenames) > 0
        assert len(filenames) == len(sizes)

        self.filenames = filenames
        self.sizes = sizes
        self.mode = mode
        self.files = []

        end = 0
        for s in self.sizes:
            end += s
        self.end = end

        self.pos = 0
        # current index for filenames, size, files
        self.current_index = 0
        self.bytes_until_end = self.sizes[self.current_index]

    def open(self, mode='r'):
        if self.files:
            return self

        open_mode = self.mode if self.mode else mode
        for fn in self.filenames:
            self.files.append(fn.open(open_mode))
        return self

    def close(self, ):
        for f in self.files:
            # TODO If a exception is raised it should continue closing
            # the files anyway
            f.close()

    def __enter__(self):
        return self.open()

    def __exit__(self, type, value, traceback):
        self.close()

    def write(self, data):
        while data:
            assert self.pos < self.end
            assert self.bytes_until_end >= 0
            if self.bytes_until_end == 0:
                self.current_index += 1
                self.bytes_until_end = self.sizes[self.current_index]
            max_write_length = self.bytes_until_end
            write_length = min(len(data), max_write_length)
            self.files[self.current_index].write(data[:write_length])
            self.pos += write_length
            self.bytes_until_end -= write_length
            data = data[write_length:]


class InterleavedFiles:
    def __init__(self, filenames, size, mode, end=None):
        assert len(filenames) > 0
        assert size > 0
        assert end is None or end % (len(filenames) * size)

        self.filenames = filenames
        self.size = size
        self.mode = mode
        self.end = end
        self.files = []

        self.pos = 0

    def open(self, mode=None):
        if self.files:
            return self

        open_mode = self.mode if self.mode else mode
        for fn in self.filenames:
            self.files.append(fn.open(open_mode))
        return self

    def close(self):
        for f in self.files:
            if self.end:
                f.seek(self.end // len(self.files))
            # TODO If a exception is raised it should continue closing
            # the files anyway
            f.close()

    def __enter__(self):
        return self.open()

    def __exit__(self, type, value, traceback):
        self.close()

    def write_simple(self, data):
        while data:
            assert self.end is None or self.pos < self.end
            file_index = self.pos // self.size % len(self.files)
            max_write_length = self.size - (self.pos % self.size)
            write_length = min(len(data), max_write_length)
            self.files[file_index].write(data[:write_length])
            self.pos += write_length
            data = data[write_length:]

    def write(self, data):
        assert self.end is None or self.pos < self.end
        chunk_offset = self.pos % (self.size * len(self.files))
        stride = self.size * (len(self.files) - 1)
        end = len(data)

        for file_index, f in enumerate(self.files):
            start_index = chunk_offset // self.size
            first_max_read_length = self.size - (chunk_offset % self.size)
            if (start_index) == file_index:
                offset = 0
            elif (start_index < file_index):
                offset = ((file_index - start_index - 1) * self.size
                          + first_max_read_length)
                first_max_read_length = self.size
            elif (start_index > file_index):
                offset = ((len(self.files) - start_index - 1) * self.size
                          + first_max_read_length + file_index * self.size)
                first_max_read_length = self.size

            buf = bytearray()
            read_end = offset + first_max_read_length
            buf.extend(data[offset:read_end])
            offset = read_end + stride
            while end >= offset:
                read_end = offset + self.size
                read_end = min(read_end, end)
                buf.extend(data[offset:read_end])
                offset = read_end + stride
            self.files[file_index].write(buf)

        self.pos += end
