#!/usr/bin/env python3
# SPDX-License-Identifier: CC0-1.0
from pathlib import Path
import os
import shutil

from convert import config
from convert.tools import (deinterleave16, split_ranges, split, mkempty,
                           deinterleave32)

from convert.graphics import pixel_de4_to_rai4, pixel_de5_to_spi6


def mkflash(target, outdir):
    target.seek(-1, os.SEEK_END)
    region = target.read(1)
    fn = outdir / f'flash0_blank_region{region[0]:02x}.u1053'
    with open(fn, 'wb+') as o:
        o.write(region)
        for _ in range(0x0F_FFFF):
            o.write(b'\xFF')


def raidenb(outdir):
    raidenb = outdir / 'raidenb'
    raidenb.mkdir(exist_ok=True)

    with open('raiden/raiden_maincpu.bin', 'rb') as f:
        maincpu_parts_low = [
            raidenb / '1.u0253',
            raidenb / '2.u0252',
        ]
        maincpu_parts_high = [
            raidenb / '3__(raidenb).u022',
            raidenb / '4__(raidenb).u023',
        ]
        deinterleave16(f, maincpu_parts_low, 0x01_0000)
        deinterleave16(f, maincpu_parts_high)

    with open('raiden/raiden_subcpu.bin', 'rb') as f:
        sub_parts = [
            raidenb / '5__(raidenb).u042',
            raidenb / '6__(raidenb).u043',
        ]
        deinterleave16(f, sub_parts)

    with open('raiden/raiden_audiocpu.bin', 'rb') as f:
        audiocpu_parts = [
            (raidenb / 'rai6.u212', [(0, 0x8000),
                                     (0x01_0000, 0x01_8000)]),
        ]
        split_ranges(f, audiocpu_parts)

    with open('raiden/raiden_gfx1.bin', 'rb') as f:
        gfx1_parts = [
            (raidenb / '9', 0x8000),
            (raidenb / '10', 0x8000)
        ]
        split(f, gfx1_parts)

    with open('raiden/raiden_gfxdata2.bin', 'rb') as f:
        gfx2 = raidenb / 'sei420'
        pixel_de4_to_rai4(f, gfx2)

    with open('raiden/raiden_gfxdata3.bin', 'rb') as f:
        gfx3 = raidenb / 'sei430'
        pixel_de4_to_rai4(f, gfx3)

    with open('raiden/raiden_gfxdata4.bin', 'rb') as f:
        gfx4 = raidenb / 'sei440'
        pixel_de4_to_rai4(f, gfx4)

    shutil.copyfile('raiden/raiden_okim6295.bin', raidenb / '7.u203')

    # plds for 0.160 and 0.174
    mkempty(raidenb / 'ep910pc-1.bin', 0x0884)
    mkempty(raidenb / 'ep910pc-2.bin', 0x0884)

    # proms for latest mame
    mkempty(raidenb / 'jj3010.u0116', 0x0100)


def rdft(outdir):
    rdft = outdir / 'rdft'
    rdft.mkdir(exist_ok=True)

    with open('rdft/rdft_i386.bin', 'rb') as f:
        maincpu_parts = [
            rdft / 'gd_1.211',
            rdft / 'gd_2.212',
            rdft / 'gd_3.210',
            rdft / 'gd_4.29'
        ]
        deinterleave32(f, maincpu_parts)

    with open('rdft/rdft_gfxdata1.bin', 'rb') as f:
        pixel_de5_to_spi6(f, rdft / 'rdft_gfxdata1.bin.spi', fix0=True,
                          fix1=True)

    with open('rdft/rdft_gfxdata2.bin', 'rb') as f:
        pixel_de5_to_spi6(f, rdft / 'rdft_gfxdata2.bin.spi')

    with open('rdft/rdft_gfxdata3.bin', 'rb') as f:
        pixel_de5_to_spi6(f, rdft / 'rdft_gfxdata3.bin.spi')

    with open('rdft/rdft_soundrom.bin', 'rb') as f:
        sound01_parts = [
            (rdft / 'gd_pcm.217', 0x20_0000),
            (rdft / 'gd_8.216', 0x08_0000),
        ]
        split(f, sound01_parts)

    with open(rdft / 'gd_1.211', 'rb') as f:
        mkflash(f, rdft)


def rdft2(outdir):
    rdft2 = outdir / 'rdft2'
    rdft2.mkdir(exist_ok=True)

    with open('rdft2/rdft2_i386.bin', 'rb') as f:
        maincpu_parts = [
            rdft2 / 'prg0.tun',
            rdft2 / 'prg1.bin',
            rdft2 / 'prg2.bin',
            rdft2 / 'prg3.bin'
        ]
        deinterleave32(f, maincpu_parts)

    with open('rdft2/rdft2_soundrom.bin', 'rb') as f:
        sound01_parts = [
            (rdft2 / 'pcm.u0217', 0x20_0000),
            (rdft2 / 'sound1.u0222', 0x08_0000),
        ]
        split(f, sound01_parts)

    with open(rdft2 / 'prg0.tun', 'rb') as f:
        mkflash(f, rdft2)


def convert():
    outdir = Path('mame')
    outdir.mkdir(exist_ok=True)

    raidenb(outdir)
    if config["partial"]:
        rdft(outdir)
        rdft2(outdir)


if __name__ == '__main__':
    convert()
